#ifndef _ADAPTIV_H_
#define _ADAPTIV_H_
#include <cmath> 

void trapez(double xa, double xb,char type,double *value)
// Trapezregel fuer Intervall [xa,xb]
{  
  double h,f1,f2;
  h=xb-xa;
  evalfunction(xa,type,&f1);
  evalfunction(xb,type,&f2); 
  *value=h/2.*(f1+f2); 
}

void errorest(double xa, double xb,char type,double *est)
// Fehlerschaetzer nach Romberg == Simpson fuer Intervall [xa,xb]
{  
  double h,fa,fb,xm,fm;
  h=xb-xa;
  xm=(xa+xb)/2.;
  evalfunction(xa,type,&fa);
  evalfunction(xb,type,&fb); 
  evalfunction(xm,type,&fm); 
  *est=h/6.*(fa+fb+4.*fm); 
}

void integrate(int m,double* mesh,int* mark,double* lokint,double* err,char type)
// berechnet lokale Integrale und den lokalen Fehler auf allen neuen
// Intervallen
{
  int i;
  for (i=0;i<m;i++)
    {
      if (mark[i]>0)
	{
	  mark[i]=0;
          trapez(mesh[2*i],mesh[2*i+1],type,&lokint[i]);
          errorest(mesh[2*i],mesh[2*i+1],type,&err[i]);
          err[i]=fabs(err[i]-lokint[i]);
        }

    }
}

void markelem(int m,int* mark,double* error)
// Markiert Intervalle mit grossem Fehler
{  
   int i;
   double gamma,errmax;
   vdmax(error,m,&errmax,&i);
   gamma=0.125;
   for (i=0;i<m;i++)
     {
       if (error[i]>gamma*errmax)
         {
	   mark[i]=1;
         }   
     }   
}

void refine(int *m,double* mesh,int* mark)
// Verfeinerung aller markierten Intervalle (mesh) durch teilung
// m.. Anzahl der Intervalle
{ int i,n,k;
  double xa,xb,xm;
  n=2*(*m);
  k=*m;
  for (i=0;i<n/2;i++)
    {
      if (mark[i]>0)
        {
          xa=mesh[2*i];
          xb=mesh[2*i+1];
          xm=(xa+xb)/2.;
          mesh[2*i+1]=xm;
          mesh[2*k]=xm;
          mesh[2*k+1]=xb;
          mark[k]=1;
	  k++;
	}
    }
  *m=k;
}

void adaptiv(double* mesh,int *m,double *Integral,double eps,char type,int nmax)
// Berechnung des Integrals
// Output: Integral und Netz
// Input: eps Genauigkeit,
// nmax maximaler Speicherplatz
// Speicherung des Netzes: mesh[2*i],mesh[2*i+1] sind Koordinaten des i-ten 
//                          Intervalls 

// Abbruch: a) genauigekit erreicht
//          b) maximaler Speicherplatz ist erreicht 
{ 
  int k,i,j;
  double* lokint;
  double* error;
  double* e;
  int* mark;  
  double sigma,value,eins;
  // Speicher allokieren
  lokint=(double*)malloc(nmax*sizeof(double));  
  error=(double*)malloc(nmax*sizeof(double));
  e=(double*)malloc(nmax*sizeof(double));
  eins=1.;
  
  mark=(int*)malloc(nmax*sizeof(int));  
  // 1. Netz
  k=1; 
  mesh[0]=0.;
  mesh[1]=1.;
  mark[0]=0;
  trapez(0.,1.,type,&value);
  lokint[0]=value;
  errorest(0.,1.,type,&value);
  error[0]=value;
  sigma=fabs(lokint[0]-error[0]);
  e[0]=1;
  // Die adaptive Schleife
  while ((sigma>eps)&&(2*k<nmax))
    {
      markelem(k,mark,error);
      refine(&k,mesh,mark);
      integrate(k,mesh,mark,lokint,error,type);
      vdcopy(k,e,1,&eins,0);
      dscapr(&sigma,k,e,error);
      cout <<"Fehlerschaetzer "<< sigma<<" bei "<<k<<" Intervallen"<<endl;
    }
  dscapr(&sigma,k,e,lokint);
  *Integral=sigma;
  *m=k; 
}

#endif
